from io import BytesIO
from zipfile import ZipFile
from urllib.request import urlopen
from pathlib import Path
from shutil import rmtree
from yaml import safe_load


class Fontawesome:
    """
    Class wrapper to deal with fontawesome based icons
    """

    FONT_VERSION = "5.15.4"
    ASSET = "https://github.com/FortAwesome/Font-Awesome/releases/download/{version}/fontawesome-free-{version}-{type}.zip"
    TYPES = {"html": "web", "latex": "desktop"}

    icons_metadata = None
    dir = None

    def download_asset(self, format, path):

        type = self.TYPES[format]
        asset = self.ASSET.format(version=self.FONT_VERSION, type=type)

        self.dir = path / "fontawesome"
        self.dir.mkdir(exist_ok=True)
        rmtree(self.dir)

        # read the zip
        resp = urlopen(asset)
        zip_file = ZipFile(BytesIO(resp.read()))
        for file in zip_file.namelist():

            if not Path(file).suffix in [".css", ".js"]:
                continue

            # get the data
            data = zip_file.read(file)

            # create the appropriate folder if needed
            src_name = Path(file).name
            src_dir = Path(*Path(file).parts[1:-1])
            dst_dir = self.dir / src_dir
            dst_dir.mkdir(exist_ok=True, parents=True)
            dts_file = dst_dir / src_name
            dts_file.write_bytes(data)

    def get_metadata(self):
        """read the yaml file to create a datatable of existing icons. Return the stored table if existing"""

        if self.icons_metadata is None:
            with (self.dir / "metadata/icons.yml").open("r") as f:
                self.icons_metadata = safe_load(f)

        return self.icons_metadata

    def get_css(self):
        """returns the complete path to the css file from _static folder"""

        return "../_font/fontawesome/css/all.min.css"

    def get_js(self):
        """returns the complete path to the js file from _static folder"""

        return "../_font/fontawesome/js/all.min.js"



# -*- coding: utf-8 -*-
from pathlib import Path
import re

from docutils import nodes
from .font_handler import Fontawesome
from sphinx.util import logging

# -- global variables ----------------------------------------------------------
font_handler = None
logger = logging.getLogger(__name__)
# ------------------------------------------------------------------------------


class icon(nodes.General, nodes.Element):
    pass


def download_font_assets(app):

    # start the font_handler
    font_handler = Fontawesome()

    # create a _font folder
    output_dir = Path(app.outdir)
    font_dir = output_dir / "_font"
    font_dir.mkdir(exist_ok=True)
    app.config.html_static_path.append(str(font_dir))

    # guess what need to be installed
    # based on the compiler
    if app.builder.format == "html":

        font_handler.download_asset("html", font_dir)
        app.add_css_file(font_handler.get_css())
        app.add_js_file(font_handler.get_js())

    elif app.builder.format == "latex":

        font_handler.download_asset("latex", font_dir)

    return


def get_glyph(text):
    """
    get the glyph from text

    Return a tuple of (glyph, font) from the provided text. raise an error if one of them does not exist

    :param text: The text to transform (e.g. "fa fa-folder")
    """

    # split the icon name to find the name inside
    m = re.match(r"^(fab|far|fa|fas) fa-([\w-]+)$", text)
    if not m:
        raise ValueError(f'invalid icon name: "{text}"')
    # if not m.group(2) in font_handler.get_metadata():
    #    raise ValueError(f'icon "{m.group(2)}" is not part of fontawesome 5.15.4')

    # return (font, glyph)
    return m.group(1), m.group(2)


def depart_icon_node(self, node):
    """Empty depart function, everything is handled in visit"""
    pass


def visit_icon_node_html(self, node):
    """create the html output"""

    try:
        font, glyph = get_glyph(node["icon"])
    except ValueError as e:
        logger.warning(str(e), location=node)
        raise nodes.SkipNode

    self.body.append(f'<i class="{font} fa-{glyph}"></i>')

    return


def visit_icon_node_latex(self, node):
    """create the latex output"""

    try:
        font, glyph = get_glyph(node["icon"])
    except ValueError as e:
        logger.warning(str(e), location=node)
        raise nodes.SkipNode

    # detect the font
    font_list = {"fa": None, "far": "regular", "fas": "solid", "fab": "brand"}
    font = font_list[font]

    # install fontawesome 5 package
    # TODO install it on the fly using the otf files downloaded in var
    package = "\\usepackage{fontawesome5}"
    if package not in self.elements["preamble"]:
        self.elements["preamble"] += f"{package}\n"

    # build the output
    cmd = "\\faIcon"
    if font is not None:
        cmd += f"[{font}]"
    cmd += f"{{{glyph}}}"

    self.body.append(cmd)

    return


def visit_icon_node_unsuported(self, node):
    """raise error when the requested output is not supported"""

    logger.warning("Unsupported output format (node skipped)")
    raise nodes.SkipNode


_NODE_VISITORS = {
    "html": (visit_icon_node_html, depart_icon_node),
    "latex": (visit_icon_node_latex, depart_icon_node),
    "man": (visit_icon_node_unsuported, None),
    "texinfo": (visit_icon_node_unsuported, None),
    "text": (visit_icon_node_unsuported, None),
}


def icon_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
    """
    add inline icons

    Returns 2 part tuple containing list of nodes to insert into the
    document and a list of system messages.  Both are allowed to be
    empty.

    :param name: The role name used in the document.
    :param rawtext: The entire markup snippet, with role.
    :param text: The text marked with the role.
    :param lineno: The line number where rawtext appears in the input.
    :param inliner: The inliner instance that called us.
    :param options: Directive options for customization.
    :param content: The directive content for customization.
    """

    # create the node
    node = icon(icon=text)

    return [node], []



__version__ = "0.1.1"
__author__ = "Pierrick Rambaud"
__email__ = "pierrick.rambaud49@gmail.com"

from . import icon


def setup(app):
    """Install the plugin.

    :param app: Sphinx application context.
    """

    # download the font to the output folder
    app.connect("builder-inited", icon.download_font_assets)

    # create the node
    app.add_node(icon.icon, **icon._NODE_VISITORS)

    # create the role
    app.add_role("icon", icon.icon_role)

    return



import setuptools
from setuptools.command.develop import develop
from subprocess import check_call


class DevelopCmd(develop):
    def run(self):
        """overwrite run command to install pre-commit hooks in dev mode"""
        check_call(["pre-commit", "install", "-t", "pre-commit", "-t", "commit-msg"])
        super().run()

        return


# set the version number
version = "0.1.1"

# set some text as CONST variables for readability
DESCRIPTION = "A sphinx custom role to embed inline fontawesome incon in the latex and html outputs"
LONG_DESCRIPTION = open("README.rst").read()

setup_params = {
    # metadata
    "name": "sphinx-icon",
    "version": version,
    "license": "BSD 2-Clause",
    "description": DESCRIPTION,
    "long_description": LONG_DESCRIPTION,
    "long_description_content_type": "text/x-rst",
    "author": "Pierrick Rambaud",
    "author_email": "pierrick.rambaud49@gmail.com",
    "url": "https://github.com/12rambau/sphinx-icon",
    "download_url": f"https://github.com/12rambau/sepal_ui/archive/v{version}.tar.gz",
    "keywords": ["skeleton", "Python"],
    "classifiers": [
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    # options
    "python_requires": ">=3.6.9",
    "packages": setuptools.find_packages(),
    "include_package_data": True,
    "namespace_packages": ["sphinxcontrib"],
    "install_requires": ["Deprecated", "docutils", "pyyaml"],
    "cmdclass": {"develop": DevelopCmd},
    # extras_require
    "extras_require": {
        "dev": ["pre-commit", "commitizen"],
        "test": ["coverage", "pytest"],
        "doc": [
            "Sphinx",
            "sphinxcontrib-spelling",
            "sphinx-copybutton",
            "furo",
        ],
    },
}

setuptools.setup(**setup_params)
